 /****************************************************************************
Date: 8/31/99
File: cmd.cpp
Description:	InstantTune TSR Command Interpreter

	This module interprets commands sent via INT63 and INT64 and
	executes them by calling the appropriate member functions of
	the CONTROLLER object.

	This module uses INT64H exclusively as its RadioDRV tuning
	command interface.

	It may also (optionally) hook the OrbitDRV interrupt, INT63H
	to automatically set the radio configuration to match the current
	satellite when tracking is enabled from InstantTrack.

	The interrupt numbers for tuning commands and OrbitDRV are settable 
	using environment variables ITRAD and ITORB which take decimal 
	values. Setting ITORB to 0 prevents use of the OrbitDRV
	interrupt and disables automatic tuning control from InstantTrack.

	This module hooks the dos timer interrupt (INT1CH) to provide
	timing services needed by the CONTROLLER and INTERVALTIMER objects.

History:
	2/20/99 added debug stuff
	6/6/99 fixed tuning status display in InstantTrack problem
	8/31/99 clean-up for release

*****************************************************************************/
// Optimizations 
#pragma optimize("c e g l", off)

#include <stdlib.h>
#include <string.h>
#include <dos.h>

#include "cmd.hpp"	// command interpreter definitions
#include "dospc.h"
#include "istsr.hpp"	// IsTSRLoaded() utility
#include "inttimer.hpp"	// INTERVALTIMER class
#include "control.hpp"	// radio tuning controller
#include "version.hpp"	// Version string

//***** DEBUG ***********************************************************
DEBUG	oDebug;		// debug object allows printing debug messages

//****** InstantTune Stack ***********************************************
//
// In order to prevent overflow of the user's application stack,
// InstantTune has its own stack. All InstantTune interrupt service
// routines share the same stack. The "unStackNest" variable holds
// the current ISR nest level. The first ISR switches to the InstantTune
// stack and the last ISR switches back. The ISR must get its machine
// register parameters BEFORE switching the stack and must set its return
// values AFTER restoring the stack.
//
// Note that the InstantTune stack is the "top" of the TSR.
//************************************************************************
static CHAR HUGEPTR pStack;	// Stack for InstantTune ISR's
static CHAR NEAR*   pAppBase;	// Saved current application base pointer (BP)
static CHAR HUGEPTR pAppStack;  // Saved current application stack pointer
static volatile USHORT unISRNest = 0;	// ISR count on InstantTune stack

#define SWITCHSTACK() __asm {				\
		__asm mov  WORD PTR pAppBase, bp	\
		__asm mov  WORD PTR pAppStack[0], sp	\
		__asm mov  WORD PTR pAppStack[2], ss	\
		__asm mov  ax, bp			\
		__asm sub  ax, sp			\
		__asm mov  bp, WORD PTR pStack[0]	\
		__asm mov  sp, WORD PTR pStack[0]	\
		__asm sub  sp, ax			\
		__asm mov  ss, WORD PTR pStack[2]	\
}

#define RESETSTACK() __asm {				\
		__asm mov  ss, WORD PTR pAppStack[2]	\
		__asm mov  sp, WORD PTR pAppStack[0]	\
		__asm mov  bp, WORD PTR pAppBase	\
}

//***** Common Objects ******************
// These objects are used by multiple
// routines in this file.
static INTHANDLER	pfnOrbitDRV;	// pointer to "real" OrbitDRV TSR
static INTHANDLER       pfnDosTimer;    // pointer to "real" DOS timer ISR
static CONTROLLER	Controller;	// Radio Tuning Controller


//***** Interrupt Service Routines *****
static VOID INTERRUPT RadioDRVISR(INTREGS);
static VOID INTERRUPT OrbitDRVISR(INTREGS);
static VOID INTERRUPT DosTimerISR(VOID);


//****** RadioDRV Signature Header *****
static const CMDSIGSTRUCT RadioDRV = { 
	0x90,			// NOP instruction (for word alignment)
	0xEA,			// x86 Long Jump instruction
	(INTHANDLER) RadioDRVISR,
	{ 0 },			// unused pad
 	CMD_SIG_RADIODRV	// RadioDRV signature
  };

//***** RadioDRV ISR *******************
VOID INTERRUPT RadioDRVISR(INTREGS) {

        // Copy register values from stack
	static INTREG regAX;
	static INTREG regBX;
	static INTREG regDX;
	static INTREG regDS;

	regAX = INTREG_AX;
	regBX = INTREG_BX;
	regDX = INTREG_DX;
	regDS = INTREG_DS;

	// If we are not already using the TSR stack, save the running
	// applications stack pointer and change to TSR stack
	if (unISRNest++ == 0) {
		SWITCHSTACK()
	}
       	ENABLE();						

        switch(regAX >> 8) {    // commands are sent in AH register

	case CMD_RADIODRV_SETVELOCITY:

		// update satellite velocity, DS:DX points to new value
		Controller.SetVelocity( *((DOUBLE FAR*) MK_FP(regDS,regDX)) );

		break;

	case CMD_RADIODRV_SETCONFIG:
		//*********************************************************
		// Set a new satellite and radio configuration.
		// DS:DX points to the satellite name.
		// Controller looks up config in itune.cfg file
		//*********************************************************
		regAX = Controller.SetConfig( 
				(const CHAR FAR*) MK_FP(regDS,regDX) );
		break;

	case CMD_RADIODRV_STOP:		// Stop Radio Tuning
		Controller.Halt();
		regAX = BOOL_FALSE;
		break;

	case CMD_RADIODRV_GETSTATUS:	// Get Tuning Status
		regAX = Controller.GetStatus();
		break;

	case CMD_RADIODRV_DEBUG:
		oDebug << "Software Version " << pVersionString; 
		Controller.Debug();		// hook only, does nothing
		regAX = 0;
		regBX = 0;
		regDX = 0;
		break;

	default:
		break;
	} // end switch()

	// restore stack
	DISABLE();						
	if (--unISRNest == 0) {
		RESETSTACK()
	}

	// set register return values
	INTREG_AX = regAX;

	return;
}


//****** OrbitDRV Signature Header *****
static const CMDSIGSTRUCT OrbitDRV = {
	0x90,			// NOP instruction (for word alignment)
	0xEA,			// x86 Long Jump instruction to ISR
	(INTHANDLER) OrbitDRVISR,
	{ 0 },			// unused pad
	{ CMD_SIG_ORBITDRV }	// OrbitDRV signature
};

//***** ORBITDRV ISR *******************
VOID INTERRUPT OrbitDRVISR(INTREGS) {

	static INTREG regAX;
	static INTREG regDX;
	static INTREG regDS;

	static CMDORBITDRVSATINFO FAR* pSatElements;

        // Copy register values from stack
	regAX = INTREG_AX;
	regDX = INTREG_DX;
	regDS = INTREG_DS;

	if (unISRNest == 0) {
		SWITCHSTACK()
	}
	unISRNest++;
       	ENABLE();						


	// Decode command sent in AH register
	switch(regAX >> 8) {

	case CMD_ORBITDRV_SETSATELLITE: 	// Set Satellite elements
		pSatElements = (CMDORBITDRVSATINFO FAR*) MK_FP(regDS,regDX);
		Controller.SetConfig(pSatElements->szName);
		break;

	case CMD_ORBITDRV_SETTRACKING: 	// start/stop tracking

		//**************************************************
		// if command is to enable tracking, check tuning controller
   		// status and if OK, send "enable tuning" to OrbitDRV.
 		// if not OK, send "disable tuning" to OrbitDRV.
		//
		// if command is to disable tracking, then stop tuning
		// controller and send "disable tuning" to OrbitDRV.
		//**************************************************

		regAX &= 0x00ff; // clear AH leaving only AL
				 // Enable => AL=1
				 // Disable => AL=0

		if (regAX) {
			// Enable tracking received
			if (Controller.GetStatus() == BOOL_FALSE) {
				regAX = 0;
			}
		}
		else {
			// Disable tracking received.
			Controller.Halt();
		}

		// Make tuning command for OrbitDRV,
		// set actual register, and call Orbitdrv
		regAX |= (CMD_ORBITDRV_SETTUNING << 8); // add opcode to AH
		__asm mov ax, regAX
		(*pfnOrbitDRV)();

		break;

	default:
		break;
	} // end switch()

	// restore stack
	DISABLE();						
	if (--unISRNest == 0) {
		RESETSTACK()
	}

	// Now chain to Orbitdrv with original registers
	_chain_intr((INTHANDLER) pfnOrbitDRV);
}

//***** DOS Timer interrupt handler ***************************************
// Note: This routine gets called every ~55mSec. This routine runs
// with the interrupts OFF.
VOID INTERRUPT DOSTimerISR(VOID) {

	if (unISRNest++ == 0 ) {
		SWITCHSTACK()
        }

	// Update Interval Timers
	INTERVALTIMER::Update();

        // Run Controller background processing routine
	Controller.Process();

	if (--unISRNest == 0 ) {
		RESETSTACK()
        }

        // let DOS timer handler run
        _chain_intr(pfnDosTimer);
}


//***** main routine *******************
// This loads the TSR

main(){
	// Initialize pStack to point to TSR stack
	__asm {
        	mov  WORD PTR pStack[0], sp
        	mov  WORD PTR pStack[2], ss
	}

	// Get pointer to bottom of TSR
	CHAR HUGEPTR	pTSRBottom;
    	FP_SEG( pTSRBottom ) = _psp;
    	FP_OFF( pTSRBottom ) = 0;

    	// Program size is: top of stack - bottom of TSR
	// converted to paragraphs + one extra paragraph 
        UINT unTSRSize = (UINT) ((pStack - pTSRBottom) >> 4) + 1;

	//***** Get interrupt numbers ***********
	// First check environment variables for
	// overrides, otherwise use defaults
	UINT	unTuneInterrupt;
	CHAR*	pszTuneInterrupt = getenv(CMD_VAR_RADIODRV);
	if (pszTuneInterrupt) {
		unTuneInterrupt = (UINT) strtoul(pszTuneInterrupt, NULL, 16);
	}
	else {
		unTuneInterrupt = CMD_INT_RADIODRV;
	}

	UINT	unOrbitDRVInterrupt;
	CHAR*	pszOrbitDRVInterrupt = getenv(CMD_VAR_ORBITDRV);
	if (pszOrbitDRVInterrupt) {
		unOrbitDRVInterrupt = (UINT)
				strtoul(pszOrbitDRVInterrupt, NULL, 16);
	}
	else {
		unOrbitDRVInterrupt = CMD_INT_ORBITDRV;
	}

	// Make sure Tuning interrupt (INT64) is unused
        // DO NOT set tune interrupt vector yet
	if (getvect(unTuneInterrupt) != NULL) {

		// Vector already used, do not load TSR
		exit(CMD_RET_VECTORINUSE);
	}


	// If in automatic mode (OrbitDRV interrupt not set to 0),
	// make sure OrbitDRV TSR is loaded and then hook interrupt
	if ( unOrbitDRVInterrupt ) {
		if ( !IsTSRLoaded(unOrbitDRVInterrupt,
				  CMD_SIG_OFFSET,
				  CMD_SIG_ORBITDRV) ) {

			exit(CMD_RET_NOORBITDRV); // Whoops no OrbitDRV!
		}
		else {
			// Hook OrbitDRV interrupt
			pfnOrbitDRV = getvect(unOrbitDRVInterrupt);
			setvect(unOrbitDRVInterrupt, (INTHANDLER) &OrbitDRV);
		}
	}

	// Now set RadioDRV interrupt
	setvect(unTuneInterrupt, (INTHANDLER) &RadioDRV);

	// Hook DOS Timer Interrupt
	disable();
        pfnDosTimer = getvect(CMD_INT_DOSTIMER);
	setvect(CMD_INT_DOSTIMER, DOSTimerISR);
	enable();

// Free the PSP segment and 
//_dos_freemem(_psp);

	// Terminate with program resident.
    	keep(CMD_RET_SUCCESS, unTSRSize);

    	return 0; // never executed

} // end cmd.cpp
