/**************************************************************************
date: 1/26/2002
file: comdrv.cpp

title:          COMDRV - PC COM Port Driver    
  
public:

description:

history: AJM initial version
	 7/25/99 fix PIC2 lockup bug
	 8/31/99 clean-up for release
	 1/26/2002 Allow COM port sharing (for ICOM)

**************************************************************************/
#include "comdrv.hpp"

#pragma check_stack( off )
#pragma check_pointer( off )

//***** Array of pointers to COMDRV objects ****************
COMDRV* COMDRV::aThisPointer[COMDRV_MAXPORTS] = { NULL };

//***** Com Port Interrupt Handlers ************************
static VOID INTERRUPT ComISR0() { COMDRV::GetThisPointer(0)->IntHandler(); }
static VOID INTERRUPT ComISR1() { COMDRV::GetThisPointer(1)->IntHandler(); }
static VOID INTERRUPT ComISR2() { COMDRV::GetThisPointer(2)->IntHandler(); }
static VOID INTERRUPT ComISR3() { COMDRV::GetThisPointer(3)->IntHandler(); }

//***** Addresses of Com port interrupt vectors ************
const INTHANDLER COMDRV::aVector[COMDRV_MAXPORTS] = {
        (INTHANDLER) ComISR0,
        (INTHANDLER) ComISR1,
        (INTHANDLER) ComISR2,
        (INTHANDLER) ComISR3
};


/****************************************************************************
Function: COMDRV::IntHandler()
Description: 	Common ComPort Interrupt Handler function
	This functions handles interrupts for all com ports. Each ISR
        calls this function with the associated "this" Pointer.
Parameters:
Returns:
*****************************************************************************/
VOID    COMDRV::IntHandler() {

        INT     nChar;          	// Temp store for data bytes

	UARTINTID IntId = Uart.GetInterruptID();	// Get interrupt ID
        do {
        	switch( IntId ) {

                case UARTINTID_THRE:	// Tx Holding Reg empty
			nChar = TxFifo.Get();
			if (nChar != COMDRV_EOF) {
				Uart.PutData(nChar);
			}
			else {
				eTxBusy = BOOL_FALSE;
			}
                        break;

                case UARTINTID_RDR:	// Rx Data Ready
                        RxFifo.Put(Uart.GetData());
                        break;

                case UARTINTID_MODEMSTATUS:
                        if ( Uart.IsCTS() ) {
                                // If CTS is High, enable THRE interrupt
                                Uart.EnableInt(UARTINTENABLE_THRE);
                        }
                        else {  // If CTS is Low, disable THRE interrupt
                                Uart.DisableInt(UARTINTENABLE_THRE);
                        }
                        break;

                case UARTINTID_LINESTATUS:
                        // Fill in here to process line errors or break
                        // Must also enable the interrupt in Open()
                        break;
     
                case UARTINTID_NONE:
                        // it wasn't this device so chain to old int handler 
			Pic.Chain();
                        break;

                default: // invalid interrupt id
                        // This is a hw error condition 
			Pic.EndOfInterrupt();	// Re-enable interrupts
                        return;
                }

		IntId = Uart.GetInterruptID();

        } while( IntId != UARTINTID_NONE );

	Pic.EndOfInterrupt();	// Re-enable interrupts
}


/****************************************************************************
Function: COMDRV()
Description: Construcor
Parameters:
Returns:
*****************************************************************************/
COMDRV::COMDRV() :
	eOpen(BOOL_FALSE),
	eTxBusy(BOOL_FALSE),
	TxFifo((UCHAR*) aTxBuffer, sizeof(aTxBuffer)),
	RxFifo((UCHAR*) aRxBuffer, sizeof(aRxBuffer)) {
}


/****************************************************************************
Function: Open()
Description: Open COM port
Parameters:
	UINT    unComPort,
	UINT	unBitRate,
        UINT    unDataBits,
        UARTPARITY eParity,  
        UINT    unStopBits,  
        UINT	unIRQ,
        IOADDRESS Address
Returns:
	BOOL_FALSE	failed
	BOOL_TRUE	success
*****************************************************************************/
BOOL    COMDRV::Open(   UINT    unComPort,
			UINT	unBitRate,
                        UINT    unDataBits,
                        UARTPARITY eParity,  
                        UINT    unStopBits,  
                        UINT	unIRQ,
                        IOADDRESS Address) {

	//***** Check for already Open **********************
	if (eOpen) {
		// if same COM port, OK (sharing)
		if (unComPort == unMyComPort) {
			return BOOL_TRUE;
		}
		// Not same COM number - can't share
		return BOOL_FALSE;
	}
	
	// Save Com port number
	unMyComPort = unComPort;

        //***** Get uart Base Address ***********************
	if ( (unComPort >= 1 && unComPort <= 4) &&
	     (Address == DEFAULTADDRESS) ) {	// default, get from BIOS
                Address = *( (IOADDRESS FAR*)
                                MK_FP(BIOSSEGMENT,((unComPort-1) * 2)) );
	}
        if (Address == DEFAULTADDRESS) {
                return BOOL_FALSE;     // can't find this COM port
        }

        //***** Get IRQ ************************************
	// check for valid IRQ number or COM_DEFAULTIRQ 
	if (unIRQ > 15) {
		return BOOL_FALSE;
	}
        // Default IRQ4 for com 1,3 IRQ3 for com 2,4 
	if (unIRQ == DEFAULTIRQ) {
		unIRQ = ((unComPort) & 0x01) ? 4 : 3;
	}

        //***** Save pointer to this object ****************
        // Find a space to store the pointer
        unIndex = COMDRV_MAXPORTS;              // start with an invalid #
        for (UINT i=0; i < COMDRV_MAXPORTS; i++) {
                if ( aThisPointer[i] == NULL ) {   // found an empty slot
			unIndex = i;
			break;
		}
	} 
        if (unIndex == COMDRV_MAXPORTS) {       // still invalid?
                return BOOL_FALSE;       // no slots left
	}
        aThisPointer[unIndex] = this;

        //***** Clear Tx and Rx FIFO's *********************
        TxFifo.Clear();
        RxFifo.Clear();

	//***** Set Up UART and PIC *************************

	_disable();			// Disable interrupts while setting up

	Pic.Install(unIRQ, aVector[unIndex]);

	Uart.Open(Address);
        Uart.SetLine(unBitRate, unDataBits, unStopBits, eParity);
        Uart.SetModemLeads();  		// set "normal" leads to TRUE
        Uart.ClearInterrupts();         // Clear any pending interrupts
	Uart.EnableInt( UARTINTENABLE_THRE
			| UARTINTENABLE_RDR
			| UARTINTENABLE_MODEMSTATUS);

	Pic.EndOfInterrupt();	// Reset the PIC

	_enable();	 // Re-enable interrupts

	eOpen = BOOL_TRUE;
	return BOOL_TRUE;
}


/****************************************************************************
Function: Putc()
Description:	Write a character 
Parameters:
	cChar		character to write
Returns:
*****************************************************************************/
VOID	COMDRV::Putc(CHAR c) {

	_disable();

	// If TX is busy, enqueue the char,
	// otherwise write it to the UART and set the busy flag
	if (eTxBusy) {
		TxFifo.Put(c);
	}
	else {
		Uart.PutData(c);
		eTxBusy = BOOL_TRUE;
	}
	_enable();
}

/****************************************************************************
Function: Getc()
Description: Get a receive char
Parameters:
Returns:
	character or COM_EOF if none available
*****************************************************************************/
INT	COMDRV::Getc() {

	INT	nChar;

	_disable();
	nChar = RxFifo.Get();
	_enable();
	return nChar;
}


/**************************************************************************** 
Function: Write()
Description:	Write a block of data to port
Parameters:
	pUserData	ptr to block of data to write
	unCount		number of bytes to write
Returns:
*****************************************************************************/
VOID	COMDRV::Write(const VOID* pUserData, UINT unCount) {

	CHAR* pData = (CHAR*) pUserData;
	while(unCount--) {
		Putc(*pData++);
	}
}

/****************************************************************************
Function: Read()
Description:	Read a block of data
Parameters:
	pUserBuffer	ptr to user's buffer, data is copied into user's buffer
	unMaxCount	max number of bytes to copy
Returns:
	number of bytes copied
*****************************************************************************/
INT	COMDRV::Read(VOID* pUserBuffer, UINT unMaxCount) {

	INT	nChar;
	UINT	unCount = 0;
	CHAR*	pBuffer = (CHAR*) pUserBuffer;

	while(unCount < unMaxCount) {
		nChar = Getc();
		if (nChar == COMDRV_EOF) {
			break;
		}
		*pBuffer++ = (CHAR) nChar;
		unCount++;
	}
	return unCount;
}

/****************************************************************************
Function: ClearBuffers()
Description:	Clear Tx and Rx buffers
	Useful for com link error recovery
Parameters:
Returns:
*****************************************************************************/
VOID	COMDRV::ClearBuffers() {

	_disable();
	TxFifo.Clear();
	RxFifo.Clear();
	_enable();
}

/****************************************************************************
Function: Close()
Description:	Close the com port
Parameters:
Returns:
*****************************************************************************/
VOID	COMDRV::Close() {
	
	if (eOpen) {

		Uart.Close();
		_disable();
		Pic.Restore();			// restore old interrupt
		_enable();
		aThisPointer[unIndex] = NULL;
		TxFifo.Clear();
		RxFifo.Clear();
		eOpen = BOOL_FALSE;
	}
}

/****************************************************************************
Function: ~COMDRV
Description:	Destructor
Parameters:
Returns:
*****************************************************************************/
COMDRV::~COMDRV() {
	Close();
}
// comdrv.cpp
