/****************************************************************************
Date: 11/23/97 EXPERIMENTAL
File: control.cpp
Description:	RADIO TUNING CONTROLLER CLASS

History:
*****************************************************************************/
#include "control.hpp"
#include "dospc.h"
#include "memmgr.hpp"
#include "cfg.hpp"	

//************************************************************ 
// Sequencer Action Functions
// return of TRUE means go to next step, FALSE means stop
//************************************************************

// Start initialization sequence
VOID	CONTROLLER::Initialize() {
	pStep = aInitialize;
 	eState = CONTROLLER_STATE_RUN;
	return;
}

// Reset sequencer to top of Tune List
BOOL	CONTROLLER::Reset() {
	pStep = aTune;
	eState = CONTROLLER_STATE_READY;
	return BOOL_FALSE;
}

// Wait for receiver
BOOL	CONTROLLER::RxWait() {
	return pRx->IsReady();
}

// Wait for 2nd receiver
BOOL	CONTROLLER::Rx2Wait() {
	return pRx2->IsReady();
}

// Wait for transmitter
BOOL	CONTROLLER::TxWait() {
	return pTx->IsReady();
}

// Update Rx info
BOOL	CONTROLLER::RxInfoPoll() {
	pRx->RefreshRxParams();
	return BOOL_TRUE;
}

// Tune the receiver
BOOL CONTROLLER::RxTune() {

	//**************************************************************
	// If user has manually retuned radio, set satellite downlink
	// to match current rx frequency. Otherwise, set rx frequency to
	// match current satellite downlink
	//**************************************************************

	if (pRx->HasUserTunedRx()) {
		pSat->SetDownlink( pRx->GetRxFreq() );
	}
	else {
		pRx->SetRxFreq( pSat->GetDownlink() );
	}
	return BOOL_TRUE;
}

// Tune the second receiver
BOOL CONTROLLER::Rx2Tune() {
	pRx2->SetRxFreq( pSat->GetDownlink2() );
	return BOOL_TRUE;
}

// Tune the transmitter
BOOL	CONTROLLER::TxTune() {
    	pTx->SetTxFreq( pSat->GetUplink() );
    	return BOOL_TRUE;
}

// Set Rx Modulation
BOOL	CONTROLLER::RxSetMode() {
	pRx->SetRxMode( pSat->GetDownlinkMode() );
	return BOOL_TRUE;
}

// Set Rx2 Modulation
BOOL	CONTROLLER::Rx2SetMode() {
	pRx2->SetRxMode( pSat->GetDownlink2Mode() );
	return BOOL_TRUE;
}

// Set Transmitter Modulation
BOOL	CONTROLLER::TxSetMode() {
	pTx->SetTxMode( pSat->GetUplinkMode() );
	return BOOL_TRUE;
}

// Get Rx Modulation
BOOL	CONTROLLER::RxGetMode() {
	pSat->SetDownlinkMode( pRx->GetRxMode() );
	return BOOL_TRUE;
}

// Initialization list
CONTROLLER::CONTROLLERACTION CONTROLLER::aInitialize[] = {

	// initialize main rx
	&CONTROLLER::RxWait,
	&CONTROLLER::RxSetMode,
	&CONTROLLER::RxWait,
	&CONTROLLER::RxTune,
	&CONTROLLER::RxWait,

	// initialize 2nd rx
	&CONTROLLER::Rx2Wait,

	// initialize tx
	&CONTROLLER::TxWait,
	&CONTROLLER::TxSetMode,
	&CONTROLLER::TxWait,
	&CONTROLLER::TxTune,
	&CONTROLLER::TxWait,

	// Reset sequencer to top of Tune list
	&CONTROLLER::Reset
};

// sequencer Tune list
CONTROLLER::CONTROLLERACTION CONTROLLER::aTune[] = {

	// get rx params
	&CONTROLLER::RxInfoPoll,
	&CONTROLLER::RxWait,

	// tune radios
	&CONTROLLER::RxTune,
	&CONTROLLER::RxWait,
	&CONTROLLER::Rx2Tune,
	&CONTROLLER::Rx2Wait,
	&CONTROLLER::TxTune,
	&CONTROLLER::TxWait,

	// set modulation
	&CONTROLLER::RxGetMode,
	&CONTROLLER::Rx2SetMode,
	&CONTROLLER::Rx2Wait,
	&CONTROLLER::TxSetMode,
	&CONTROLLER::TxWait,

	// Reset sequencer back to begining of this list
	&CONTROLLER::Reset
};

CONTROLLER::CONTROLLER() :
		eState(CONTROLLER_STATE_STOP),
		pSat(NULL),
		pRx(NULL),
		pRx2(NULL),
		pTx(NULL),
		pStep(aTune) {}


VOID CONTROLLER::SetVelocity(DOUBLE dVelocity) {
	disable();
	if (eState == CONTROLLER_STATE_READY) {
		pSat->SetVelocity(dVelocity);
		eState = CONTROLLER_STATE_RUN;
	}
	enable();
	return;
}

VOID CONTROLLER::Halt() {
	disable();
	eState = CONTROLLER_STATE_STOP;
	delete pSat;
	pSat = NULL;
	if (pTx != pRx) { delete pTx; }
	delete pRx;
	delete pRx2;
	pRx = pTx = pRx2 = NULL;
	enable();
	return;
}

BOOL CONTROLLER::SetConfig(const CHAR FAR* lpszSatName) {

	// If already running, stop tuning
	if ( GetStatus() ) {
		Halt();
	}
	
	CONFIG Server;
	CONFIGINFO* pConfig = Server.GetConfig(lpszSatName);

	if (pConfig == NULL) {

		return BOOL_FALSE;
	}

	// Create new objects
	pSat = SATELLITE::NewInstance(pConfig->SatParams);
	pRx  = RADIOSYSTEM::NewInstance(pConfig->RxParams);
	pRx2 = RADIOSYSTEM::NewInstance(pConfig->Rx2Params);
	if (pConfig->RxParams.eTransceive) {
		pTx = pRx;
	}
	else {
		pTx  = RADIOSYSTEM::NewInstance(pConfig->TxParams);
	}

	if ( !pSat || !pRx || !pRx2 || !pTx) {
		Halt(); 
		return BOOL_FALSE;
	}

	Initialize();
	return BOOL_TRUE;
}

//***** BIG NOTICE *****************************************************
// This routine MUST run with the interrupts OFF.
//**********************************************************************
VOID CONTROLLER::Process() {

	if (eState != CONTROLLER_STATE_STOP) {
	
		pRx->Update();
		pRx2->Update();
		pTx->Update();
	}
	
	if (eState == CONTROLLER_STATE_RUN) {

		while ( (this->**pStep)() ) { 
			pStep = pStep + 1;
		}

	}

	return;
}

VOID CONTROLLER::Debug() {
	if (eState != CONTROLLER_STATE_STOP) {
		pRx->Debug();
		pTx->Debug();
		pRx2->Debug();
	}
}

CONTROLLER::~CONTROLLER() {
	Halt();
}

// end control.cpp
