/****************************************************************************
Date: 3/18/2002
File: ft736.cpp
Description:
	Basic FT736 radio driver

History:
	12/3/97 Test version
	1/11/98	Add keyboard tuning
	3/18/2002 Add debug stuff

*****************************************************************************/
#include "ft736.hpp"

//DEBUG
#include "debug.hpp"
extern DEBUG oDebug;

VOID	FT736::Debug() {
	oDebug << "FT-736 DEBUG\n";
	oDebug << "lRxActualSteps = " << lRxActualSteps << "\n";
	oDebug << "lRxVFOSteps = " << lRxVFOSteps << "\n";
}

// Set Mode Table
// Index into table is MODULATION type, returns FT-736 mode code
const UCHAR FT736::aSetModeTable[] = {
	0x01,	// NA	(USB is default)
	0x00,	// LSB	
	0x01,	// USB
	0x02,	// CW
	0x08,	// FM
	0x01,	// AM	(no AM mode, use USB)
	0x01	// FSK	(no FSK mode, use USB)
};

/****************************************************************************
Function: SetFrequency()				PRIVATE
Description: Set REAL FT736 Radio to this frequency
Parameters:
	Steps = desired frequency in steps
	StepSize = size of frequency steps in Hz
	ucOpcode = opcode to use in message
Returns:
*****************************************************************************/
VOID	FT736::SetFrequency(LONG	lFrequencySteps,
			    LONG	lStepSize,
			    UCHAR	ucOpcode) {

	// Convert Steps to frequency
	LONG	lFrequency = lFrequencySteps * lStepSize;

	//********************************************
	// Convert frequency to "Yaesu BCD" and
	// save in message params buffer
	//********************************************
	LONG lDivisor	  = 100000000L; 	// (i.e. 100 MHz)
	LONG lTemp;
	UCHAR* pParam = Message.aParams;
	for (SHORT nParam = 0; nParam < FT736_MESSAGEPARAMS; nParam++) {

		// Make upper nibble
		lTemp = lFrequency / lDivisor;
		lFrequency -= (lTemp * lDivisor);
		*pParam = (UCHAR) lTemp << 4;
		lDivisor /= 10;

		// Add in lower nibble
		lTemp = lFrequency / lDivisor;
		lFrequency -= (lTemp * lDivisor);
		*pParam++ += (UCHAR) lTemp;
		lDivisor /= 10;
	}

	SendMessage(ucOpcode);
	return;
}

/****************************************************************************
Function: FT736()
Description: constructor
Parameters:
	RADIOPARAMS structure
Returns:
*****************************************************************************/
FT736::FT736(const RADIOPARAMS& Params) :
	
	// Initialize
	RADIO(),				// base class
	lRxStepSize(FT736_DEFAULTSTEP),
	lRxVFOSteps(0),
	lRxRITSteps(0),
	lRxActualSteps(0),
	eRxMode(MODULATION_NA),
	lTxStepSize(FT736_DEFAULTSTEP),
	lTxVFOSteps(0),
	lTxActualSteps(0),
	eTxMode(MODULATION_NA),
	Comport(),
	KbdTuner() {

	//********************************************************
	// Open com port and check communications with radio
	// If fails, return with Operational flag FALSE
	//********************************************************
	if (Comport.Open(
			Params.unPort,
			FT736_BITRATE,
			FT736_DATABITS,
			FT736_PARITY,
			FT736_STOPBITS,
			Params.unIRQ,
			Params.Address) == BOOL_FALSE) {

		RADIO::eOperational = BOOL_FALSE;
		return;					// failed to open
	}

	//**********************************************************
	// Check communications with Radio.
	// Send test message to Radio and start a 1 second timer.
	// If timer expires before we receive test message response,
	// communications has failed.
	//***********************************************************
/**** Disable for now until we figure out why it does not work!
	SendMessage(FT736_TEST);
	INTERVALTIMER	Timer;
	Timer.Start(1);
	while(Comport.Getc() != FT736_TEST) {
		if (Timer.IsExpired()) {
 			// Radio failed to communicate
			RADIO::eOperational = BOOL_FALSE;
			return;
		}
	}
	// Radio is talking!
*******/


	//****** Set up Radio *****************************************
	// Turn on CAT mode
	SendMessage(FT736_CATON);

	// if Full duplex, set full-duplex opcodes and set radio to this mode
	if (Params.eTransceive) {

		ucSetRxFreq = FT736_DUPLEXRXFREQ;
		ucSetRxMode = FT736_DUPLEXRXMODE;
		ucSetTxFreq = FT736_DUPLEXTXFREQ;
		ucSetTxMode = FT736_DUPLEXTXMODE;

		SendMessage(FT736_FULLDUPLEXON);
	}
	else {
		// using radio as either tx or rx but not both
		ucSetRxFreq = FT736_SETFREQ;
		ucSetRxMode = FT736_SETMODE;
		ucSetTxFreq = FT736_SETFREQ;
		ucSetTxMode = FT736_SETMODE;
	}

	KbdTuner.Activate(); 
	return;
}

/****************************************************************************
Function: 	~FT736()
Description:	destructor
Parameters:
Returns:
*****************************************************************************/
FT736::~FT736() {

	// Turn off CAT mode
	SendMessage(FT736_CATOFF);

	// Wait for CAT_OFF message 
	INTERVALTIMER	Timer;
	Timer.Start(1);
	while (!Timer.IsExpired()) { }

	// Clean-up
	Comport.Close();
	return;
}

/****************************************************************************
Function: Update()
Description:
	Update radio frequencies to current params.

	Comment: The updating of the rx VFO and RIT is done here instead
	of in RefreshRxParams() in order to provide fast response time
	to keyboard tuning.

Parameters:
	
Returns:
*****************************************************************************/
VOID	FT736::Update() {

	// Check for Transmit tuning change
	if (lTxVFOSteps != lTxActualSteps) {
		lTxActualSteps = lTxVFOSteps;
		SetFrequency(lTxVFOSteps, lTxStepSize, ucSetTxFreq);
	}

	// Update Receiver VFO and RIT steps from Keyboard tuner driver
	// Note that kbd tune step is 50Hz CW/SSB or 500Hz FM.
	lRxVFOSteps += (5 * KbdTuner.GetVFOSteps());
	lRxRITSteps += KbdTuner.GetRITSteps();
	LONG	lRxDesiredSteps = lRxVFOSteps + lRxRITSteps;
	
	// Check for Receiver tuning change
	if (lRxDesiredSteps != lRxActualSteps) {
		SetFrequency(lRxDesiredSteps, lRxStepSize, ucSetRxFreq);
		lRxActualSteps = lRxDesiredSteps;
	}
	
	return;
}

/****************************************************************************
Function: SetTxFreq()
Description:
	Set transmit frequency
Parameters:
	Frequency
Returns:
*****************************************************************************/
VOID	FT736::SetTxFreq(DOUBLE dFrequency) {
	lTxVFOSteps = FreqtoSteps(dFrequency, lTxStepSize);
	return;
}

/****************************************************************************
Function: SetTxMode()
Description:
	Set transmitter modulation mode type
Parameters:
	Mode
Returns:
*****************************************************************************/
VOID	FT736::SetTxMode(MODULATION eMode) {

	// Send message if different from current tx mode
	if (eTxMode != eMode) {
		SetMode(ucSetTxMode, eMode); // send message to radio
		eTxMode = eMode;	// Save new tx mode
		lTxStepSize = GetStepSize(eMode); // adjust frequency step
	}
	return;
}

/****************************************************************************
Function: HasUserTunedRx()
Description:
	Indicates if user has manually changed rx tuning
Parameters:
Returns:
	BOOL_FALSE if no changes
	BOOL_TRUE if user has changed tuning
*****************************************************************************/
BOOL	FT736::HasUserTunedRx() {
	return KbdTuner.HasUserTuned();
}

/****************************************************************************
Function: RefreshRxParams()
Description:
	Updates receiver modulation mode to reflect user changes via keyboard
	tuner. See comment in Update().
Parameters:
Returns:
*****************************************************************************/
VOID	FT736::RefreshRxParams() {

	// Update Receiver Mode from Keyboard tuner driver
	MODULATION eMode = KbdTuner.GetModulation();
	if (eMode != MODULATION_NA) {
		SetRxMode(eMode);
	}
	return;
}

/****************************************************************************
Function: GetRxFreq()
Description:
	Gets Receive (VFO) frequency
Parameters:
Returns:
	Frequency
*****************************************************************************/
DOUBLE	FT736::GetRxFreq() {
	return (DOUBLE) (lRxVFOSteps * lRxStepSize);
}

/****************************************************************************
Function: GetRxMode()
Description:
	Gets Receive modulation mode
Parameters:
Returns:
	mode
*****************************************************************************/
MODULATION FT736::GetRxMode() {
	return eRxMode;
}

/****************************************************************************
Function: SetRxFreq()
Description:
	Set Receive frequency
Parameters:
	Frequency
Returns:
*****************************************************************************/
VOID	FT736::SetRxFreq(DOUBLE dFrequency) {
	lRxVFOSteps = FreqtoSteps(dFrequency, lRxStepSize);
	return;
}

/****************************************************************************
Function: SetRxMode()
Description:
	Set receive modulation mode type. Note that this may change
	the tuning step size also.
Parameters:
	Mode
Returns:
*****************************************************************************/
VOID	FT736::SetRxMode(MODULATION eMode) {

	// Send message new mode is different from current mode
	if (eRxMode != eMode) {
		SetMode(ucSetRxMode, eMode); // Send message to radio
		eRxMode = eMode; 	// Save new Rx mode

		// Adjust tuning steps for new mode.
		LONG lNewStepSize = GetStepSize(eMode); 
		lRxVFOSteps = (lRxVFOSteps * lRxStepSize) / lNewStepSize;
		lRxRITSteps = (lRxRITSteps * lRxStepSize) / lNewStepSize;
		lRxStepSize = lNewStepSize;
	}
	return;
}

// end FT-736.cpp
