/****************************************************************************
Date: 5/18/99
File: ft847.hpp
Description:
	Yaseu FT-847 radio driver

History:
	12/24/98 Test version 1 (FT-736 with new opcodes)
	1/6/99 Version 2 - read VFO + mode
	3/28/99 Version 3 - add XIT ("clarifier" doesn't work right)
	5/18/99 make frequency step always 10 Hz
*****************************************************************************/
#ifndef FT847_HPP
#define FT847_HPP

#include "loctypes.h"
#include "radio.hpp"	// RADIO base class
#include "comdrv.hpp"	// COM port driver class
#include "inttimer.hpp"	// interval timer


//***** Frequency Steps ****************
const LONG FT847_STEPSIZE	= 10;	// 10 Hz frequency tuning step size

//***** Message Definitions ************
// All messages are 5 bytes
const INT FT847_MESSAGESIZE	= 5;  	// number of bytes in message

// Sent Message consists of 4 bytes for
// parameters and 1 byte for opcode.
// Opcode is always last.
const INT FT847_MESSAGEPARAMS= 4;  	// number of params in message to radio

//****** OpCodes ***********************
// Radio command codes
const UCHAR FT847_CATON		= 0x00;	// Computer-Aided Transceiver (ON)
const UCHAR FT847_CATOFF	= 0x80;	// Computer-Aided Transceiver (OFF)
const UCHAR FT847_SATMODEON	= 0x4e;	// satellite mode ON
const UCHAR FT847_SATMODEOFF	= 0x8e;	// satellite mode OFF
const UCHAR FT847_SETFREQ	= 0x01;	// Set main VFO frequency
const UCHAR FT847_SETRXFREQ	= 0x11;	// Set Rx frequency (sat mode)
const UCHAR FT847_SETTXFREQ	= 0x21;	// Set Tx frequency (sat mode)
const UCHAR FT847_SETMODE	= 0x07;	// Set main VFO modulation
const UCHAR FT847_SETRXMODE	= 0x17;	// Set Rx modulation (sat mode)
const UCHAR FT847_SETTXMODE	= 0x27;	// Set Tx modulation (sat mode)
const UCHAR FT847_GETVFO	= 0x03;	// Get main VFO frequency and mode
const UCHAR FT847_GETRXVFO	= 0x13;	// Get RX VFO freq and mode (sat mode)
const UCHAR FT847_GETTXVFO	= 0x23;	// Get TX VFO freq and mode (sat mode)
const UCHAR FT847_GETRXSTATUS	= 0xe7;	// Get Rx Status
const UCHAR FT847_GETTXSTATUS	= 0xf7;	// Get Tx Status

//***** COM PORT PARAMETERS ******************************************
const USHORT FT847_TIMEOUT	= 1;	// 1 second response timer
const UINT	FT847_BITRATE	= 4800; // default bit rate
const UINT	FT847_DATABITS	= 8;
const UINT	FT847_STOPBITS	= 2;
const UARTPARITY FT847_PARITY	= UARTPARITY_NONE;

//***** Message State ******************************************
// This type is used to record the request sent to the radio
// This is needed so that received messages are properly
// handled because messages from the FT-847 do not have an opcode.
enum FT847_MESSAGE {
	FT847_MESSAGE_NONE,
	FT847_MESSAGE_RXVFO,
	FT847_MESSAGE_TXVFO
};

class FT847 : public RADIO {
private:
	INTERVALTIMER	Timer;		// used for COM timeout
	FT847_MESSAGE	eMessage;	// expected message type from radio
	BOOL		eSatMode;	// True if using satellite mode
	BOOL		eReady;		// ready for next command

	// Radio Control OPCODES
	UCHAR		ucSetRxFreq;	// OPCODE to use to set rx freq
	UCHAR		ucSetRxMode;	// OPCODE to use to set rx mode
	UCHAR		ucSetTxFreq;	// OPCODE to use to set tx freq
	UCHAR		ucSetTxMode;	// OPCODE to use to set tx mode
	UCHAR		ucGetRxInfo;	// OPCODE to use to get rx freq & mode

	// Rx Params
	LONG		lRxVFOSteps;	// receiver VFO steps
	MODULATION	eRxMode;	// current rx modulation type
	BOOL		eUserHasTunedRx;// TRUE if user changed VFO

	// Tx Params
	LONG		lTxVFOSteps;	// last VFO steps sent to radio
	LONG		lTxXITSteps;	// sub band tuning adjust (XIT)
	MODULATION	eTxMode;	// current Tx modulation type
	
	// Com port driver
	COMDRV		Comport;
	
	// Message Buffers
	struct {
		UCHAR	aParams[FT847_MESSAGEPARAMS];
		UCHAR	ucOpcode;
	} OutputBuffer;

	UCHAR	aInputBuffer[FT847_MESSAGESIZE];
	UINT	unByteCounter;	// received bytes in input buffer
	

	//******************************************************
	// Utilities
	//******************************************************
	
	// Table to convert RADIO::MODULATION type to FT-847 message param
	static const UCHAR aSetModeTable[];

	// Table to convert FT-847 message param to RADIO::MODULATION type
	static const MODULATION aGetModeTable[];
	
	// Send a message to the radio
	VOID	SendMessage(UCHAR ucOpcode) {
		OutputBuffer.ucOpcode = ucOpcode;
		Comport.Write(&OutputBuffer, FT847_MESSAGESIZE);
		eMessage = FT847_MESSAGE_NONE;
	}

	// Set radio mode
	VOID	SetMode(UCHAR ucOpcode, MODULATION eMode) {
		OutputBuffer.aParams[0] = aSetModeTable[eMode];
		SendMessage(ucOpcode);
	}

	// Convert Frequency to steps
	LONG	FreqtoSteps(DOUBLE dFrequency) {

		//************************************************
		// WARNING: this code has a workaround for MS C7 bug
		// converting double to long. Do NOT change!
		//************************************************
		volatile ULONG ulFrequency = (ULONG) dFrequency;
		LONG lFrequency = (LONG) ulFrequency;
		return ( (lFrequency) + (FT847_STEPSIZE/2) ) / FT847_STEPSIZE;
	}

	// Set actual radio frequency
	VOID SetFrequency(LONG	lFrequencySteps,
			  UCHAR	ucOpcode);

	
	// Process GetVFO response message
	VOID	ProcessVFO(LONG& lFreqSteps, MODULATION& eMode);

	// No copy constructor or assignment allowed
	// Force compile error if attempted
	FT847(const FT847&);
	FT847& operator=(const FT847&);

public:
	// General Functions
	FT847(const RADIOPARAMS& Params);
	BOOL	IsReady() { return eReady; }
	VOID	Update();
	~FT847();

	// Tx Functions
	VOID	SetTxFreq(DOUBLE dFrequency);
	VOID	SetTxMode(MODULATION eMode);

	// Rx Functions
	BOOL	HasUserTunedRx() { return eUserHasTunedRx; }
	VOID	RefreshRxParams();
	DOUBLE	GetRxFreq();
	MODULATION GetRxMode() { return eRxMode; }
	VOID	SetRxFreq(DOUBLE dFrequency);
	VOID	SetRxMode(MODULATION eMode);

};
#endif // end ft847.hpp
