/****************************************************************************
Date: 3/9/2002
File: icom.cpp
Description:
	ICOM Radio Driver for Satellite radios (IC-970, IC-820,
	IC-821, IC-910)
History:
	6/3/2000 initial version
	8/18/2001 radio.hpp parameter name change (unParam1)
	1/10/2002 fix init problem, try again (use SWAP this time)
	2/5/2002 add IC-910
	3/3/2002 Set mode after swap
	3/9/2002 Fix tuning step error
*****************************************************************************/
#include "icom.hpp"

/****************************************************************************
Function: Constructor
Description:
Parameters: RADIOPARAMS, RadioType
Returns:
*****************************************************************************/
ICOM::ICOM(const RADIOPARAMS& Params, const USHORT unRadioType) :

	RADIO(),			// Base class
	Msg(),				// Msg Handler
	eReady(BOOL_FALSE),
	eFullDuplex(Params.eTransceive),
	ucCurrentBand(0),		// This means "unknown"
	eState(ICOM_STATE_IDLE),
	lRxVFOSteps(0),
	eRxMode(MODULATION_NA),
	eUserHasTunedRx(BOOL_FALSE),
	lTxVFOSteps(0),
	eTxMode(MODULATION_NA) {

	RADIO::eOperational = Msg.Open( Params.unParam1,	// CI-V Address
					Params.unPort,		// COM port
					Params.unRate,		// Bit rate
					Params.unIRQ,		// HW - IRQ
					Params.Address);	// HW IO address

	if (!RADIO::eOperational) {
		return;		// Msg handler failed to open COM port
	}
	
	// Special handling of IC-910
	if (unRadioType == ICOM_IC910) {
		// Set bands to use for Tx and Rx 
		ucRxBand = ICOMMSG_MAIN;
		ucTxBand = ICOMMSG_SUB;
	}
	else {
		// All other ICOM radios
		ucRxBand = ICOMMSG_SUB;
		ucTxBand = ICOMMSG_MAIN;
	}

	// Ready to Go!
	eState = ICOM_STATE_IDLE;
	eReady = BOOL_TRUE;
	return;
}

/****************************************************************************
Function: Update() 
Description:
	This is the state machine that processes all
	radio messages. It runs every clock tick.
Parameters:
Returns:
*****************************************************************************/
VOID	ICOM::Update() {

	// If READY flag is set, nothing to do
	if (eReady) {
		return;
	}

	// Check for a received message
	if (!Msg.IsReady()) {
		return;			// No message, nothing to do
	}

	// Process Message based on command type
	switch(Msg.GetCmd()) {

		LONG lNewRxVFOSteps;	// temp

	case ICOMMSG_READFREQ:
		lNewRxVFOSteps = GetSteps(Msg.GetFrequency(), eRxMode);
		if (lNewRxVFOSteps != lRxVFOSteps) {
			eUserHasTunedRx = BOOL_TRUE;
			lRxVFOSteps = lNewRxVFOSteps;
		}
		eState = ICOM_STATE_IDLE; // done refreshing rx params
		eReady = BOOL_TRUE;
	    	break;	

	case ICOMMSG_READMODE:
		eRxMode = Msg.GetMode();
		// Got the RX mode, now get the RX freq
		Msg.SendCmd(ICOMMSG_READFREQ);
		break;

	case ICOMMSG_OK:

		switch(eState) {

		case ICOM_STATE_SETTXFREQ:
			Msg.SendWriteFrequency(lTxVFOSteps * 
						GetStepSize(eTxMode));
			eState = ICOM_STATE_WAITOKTXFREQ;
			break;

		case ICOM_STATE_SETTXMODE:
			Msg.SendWriteMode(eTxMode);
			eState = ICOM_STATE_WAITOK;
			break;

		case ICOM_STATE_SETRXFREQ:
			Msg.SendWriteFrequency(lRxVFOSteps *
						GetStepSize(eRxMode));
			eState = ICOM_STATE_WAITOKRXFREQ;
			break;

		case ICOM_STATE_SETRXMODE:
			Msg.SendWriteMode(eRxMode);
			eState = ICOM_STATE_WAITOK;
			break;

		case ICOM_STATE_GETRXPARAMS:
			Msg.SendCmd(ICOMMSG_READMODE);
			break;

		case ICOM_STATE_WAITOKSWAPTX: 
			// After SWAP VFOs, need to set mode
			// Then re-send tx freq
			Msg.SendWriteMode(eTxMode);
			eState = ICOM_STATE_SETTXFREQ;
			break;

		case ICOM_STATE_WAITOKSWAPRX: 
			// After SWAP VFOs, need to resend mode
			// Then re-send rx freq
			Msg.SendWriteMode(eRxMode);
			eState = ICOM_STATE_SETRXFREQ;
			break;

		default: // all other messages - done!
			eState = ICOM_STATE_IDLE;
			eReady = BOOL_TRUE;
			break;
	    	} // end switch() 
		break; // end of ICOMMSG_OK processing

	case ICOMMSG_FAIL:	// Got a "fail" response

		switch(eState) {
		case ICOM_STATE_WAITOKTXFREQ:
			// Trying to set TX VFO - wrong band so swap BANDS
			// then try again
			Msg.SendCmd(ICOMMSG_VFOSELECT, ICOMMSG_EXCHANGE);
			eState = ICOM_STATE_WAITOKSWAPTX;
			break;

		case ICOM_STATE_WAITOKRXFREQ:
			// Trying to set TX VFO - wrong band so swap BANDS
			// then try again
			Msg.SendCmd(ICOMMSG_VFOSELECT, ICOMMSG_EXCHANGE);
			eState = ICOM_STATE_WAITOKSWAPRX;
			break;

		default: 	// Something bad happened
			eState = ICOM_STATE_IDLE;
			eReady = BOOL_TRUE;
			break; // Just ignore it and maybe it will go away!
		} // end switch()
		break; 	// end of ICOMMSG_FAIL processing

	default: // no such message, ignore
		eState = ICOM_STATE_IDLE;
		eReady = BOOL_TRUE;
		break;
	} // end msg processing

	return;
}

/****************************************************************************
Function: Destructor
Description:
Parameters:
Returns:
*****************************************************************************/
ICOM::~ICOM() {
	Msg.Close();
}


/****************************************************************************
Function: SetTxFreq()
Description:
	Sets Transmitter frequency
Parameters:
	DOUBLE 	dFrequency
Returns:
*****************************************************************************/
VOID	ICOM::SetTxFreq(DOUBLE dFrequency) {

	//***** Get new TX VFO steps *****
	LONG lFrequency = FreqtoLong(dFrequency);
	LONG lNewTxVFOSteps = GetSteps(lFrequency, eTxMode);
	if (lNewTxVFOSteps == lTxVFOSteps) {
		return;				// no change
	}
	lTxVFOSteps = lNewTxVFOSteps;		// Save New Tx Freq Steps

	if ( IsCurrentBand(ucTxBand) ) {
		Msg.SendWriteFrequency(lTxVFOSteps * GetStepSize(eTxMode));	
		eState = ICOM_STATE_WAITOKTXFREQ;
	}
	else {
		eState = ICOM_STATE_SETTXFREQ;
	}

	eReady = BOOL_FALSE;
	return;
}


/****************************************************************************
Function: SetTxMode()
Description: Set transmit modulation type
Parameters:
	MODULATION eMode
Returns:
*****************************************************************************/
VOID	ICOM::SetTxMode(MODULATION eMode) {

	if (eMode == eTxMode) { 		// if Mode has changed...
		return;
	}
	eTxMode = eMode; 		// Save New Tx Mode

	if ( IsCurrentBand(ucTxBand) ) {
		Msg.SendWriteMode(eMode);	
		eState = ICOM_STATE_WAITOK;
	}
	else {
		eState = ICOM_STATE_SETTXMODE;
	}

	eReady = BOOL_FALSE;
	return;
}

/****************************************************************************
Function: RefreshRxParams()
Description:
	Poll radio to get receiver frequency and mode
Parameters:
Returns:
*****************************************************************************/
VOID	ICOM::RefreshRxParams() {

	if ( IsCurrentBand(ucRxBand) ) {
		Msg.SendCmd(ICOMMSG_READMODE); 
	}

	eState = ICOM_STATE_GETRXPARAMS;
	eReady = BOOL_FALSE;
	return;
}

/****************************************************************************
Function: GetRxFreq()
Description:
	Returns current receiver frequency from previously
	refreshed rx params
Parameters:
Returns:
	DOUBLE	dFrequency
*****************************************************************************/
DOUBLE	ICOM::GetRxFreq() {

	// Set flag to show that user has not tuned from this returned freq
	eUserHasTunedRx = BOOL_FALSE;

	// Convert RX VFO Frequency Steps to Double
	LONG lFrequency = lRxVFOSteps * GetStepSize(eRxMode);
	return (DOUBLE) lFrequency;
}


/****************************************************************************
Function: SetRxFreq()
Description:
	Sets Receiver frequency
Parameters:
	DOUBLE 	dFrequency
Returns:
*****************************************************************************/
VOID	ICOM::SetRxFreq(DOUBLE dFrequency) {

	//***** clear user has tuned flag ****
	eUserHasTunedRx = BOOL_FALSE;

	//***** Get new RX VFO steps *****
	LONG lFrequency = FreqtoLong(dFrequency);
	LONG lNewRxVFOSteps = GetSteps(lFrequency, eRxMode);
	if (lNewRxVFOSteps == lRxVFOSteps) {
		return;				// unchanged
	}
	lRxVFOSteps = lNewRxVFOSteps; 	// Save New Rx Frequency Steps

	if ( IsCurrentBand(ucRxBand) ) {
		Msg.SendWriteFrequency(lRxVFOSteps * GetStepSize(eRxMode));	
		eState = ICOM_STATE_WAITOKRXFREQ;
	}
	else {
		eState = ICOM_STATE_SETRXFREQ;
	}

	eReady = BOOL_FALSE;
	return;
}

/****************************************************************************
Function: SetRxMode()
Description:
	Sets receiver modulation
Parameters:
	MODULATION eMode
Returns:
*****************************************************************************/
VOID	ICOM::SetRxMode(MODULATION eMode) {

	if (eMode == eRxMode) {
		return;				// unchanged
	}
	eRxMode = eMode;			// save new mode


	if ( IsCurrentBand(ucRxBand) ) {
		Msg.SendWriteMode(eMode);	
		eState = ICOM_STATE_WAITOK;
	}
	else {
		eState = ICOM_STATE_SETRXMODE;
	}

	eReady = BOOL_FALSE;
	return;
}
// end icom.cpp
