/****************************************************************************
Date: 3/9/2002
File: icom.hpp	

Description:
	ICOM radio driver for ICOM Satellite Radios
	IC-970, IC-820, IC-821, IC-910
History: 
	6/3/2000 Initial Version
	1/7/2002 fix init problem, try again (use band swap)
	2/5/2002 Add IC-910 (MAIN is Rx, SUB is Tx)
	3/3/2002 Set mode after swap and IC-910 mode codes
	3/9/2002 Change FM tuning step to 100 Hz
*****************************************************************************/
#ifndef ICOM_HPP
#define ICOM_HPP

#include "loctypes.h"	// local types
#include "radio.hpp"	// RADIO base class
#include "icommsg.hpp"	// ICOM serial port message handler

//***** Frequency Step Sizes ***********************************************
const LONG ICOM_STEPSIZE	= 10;	// 10 Hz frequency tuning step size
const LONG ICOM_FMSTEPSIZE	= 100;	// 100 Hz frequency step in FM mode

//***** Radio Identifier ***************************************************
// 	Use this for Param2 in Constructor
// 	Old style ICOMs use SUB for Rx, MAIN for Tx
// 	IC-910 uses SUB for Tx, MAIN for Rx
const USHORT ICOM_DEFAULT = 0;	// Use for IC-970, IC-820, IC-821
const USHORT ICOM_IC910 = 1;	// Use for IC-910

class ICOM : public RADIO {
private:
	ICOMMSG		Msg;		// ICOM Message handler

	BOOL		eReady;		// ready for next command

	BOOL		eFullDuplex;	// true if full duplex mode
	UCHAR		ucCurrentBand;	// For full Duplex only use
					// ICOMMSG_SUB or ICOMMSG_MAIN

	// Message Processing States
	enum	ICOM_STATE {
		ICOM_STATE_SETTXFREQ,
		ICOM_STATE_WAITOKTXFREQ,	// waiting for OK to set tx
		ICOM_STATE_SETTXMODE,
		ICOM_STATE_SETRXFREQ,
		ICOM_STATE_WAITOKRXFREQ,	// waiting for OK to set rx
		ICOM_STATE_SETRXMODE,
		ICOM_STATE_GETRXPARAMS,
		ICOM_STATE_WAITOK,		// wait ok to be done
		ICOM_STATE_WAITOKSWAPTX,	// wait OK from SWAP to set tx
		ICOM_STATE_WAITOKSWAPRX,	// wait ok from SWAP to set rx
		ICOM_STATE_IDLE
	};
	ICOM_STATE	eState;		// Message processing state

	// Rx Params
	LONG		lRxVFOSteps;	// receiver VFO steps
	MODULATION	eRxMode;	// current rx modulation type
	BOOL		eUserHasTunedRx;// TRUE if user changed VFO
	UCHAR		ucRxBand;	// opcode for Rx Band

	// Tx Params
	LONG		lTxVFOSteps;	// last VFO steps sent to radio
	MODULATION	eTxMode;	// current Tx modulation type
	UCHAR		ucTxBand;	// opcode for Tx Band
	

	//******************************************************
	// Utilities
	//******************************************************

	// Convert DOUBLE Frequency to LONG
	LONG	FreqtoLong(DOUBLE dFrequency) {

		//************************************************
		// WARNING: this code has a workaround for MS C7 bug
		// converting double to long. Do NOT change!
		//************************************************
		volatile ULONG ulFrequency = (ULONG) dFrequency;
		LONG lFrequency = (LONG) ulFrequency;
		return lFrequency;
	}

	// Get frequency step size for this mode
	LONG	GetStepSize(MODULATION eMode) {
		return (eMode == MODULATION_FM) ?
			ICOM_FMSTEPSIZE : ICOM_STEPSIZE;
	}

	// Convert Frequency and step size to number of Steps
	LONG	GetSteps(LONG lFrequency, MODULATION eMode) {
		LONG lStepSize = GetStepSize(eMode);
		return ( lFrequency + (lStepSize/2) ) / lStepSize;
	}


	// Check MAIN or SUB band, change if not the desired band
	BOOL	IsCurrentBand(UCHAR ucDesiredBand) {
		if (!eFullDuplex) {
			ucDesiredBand = ICOMMSG_MAIN;
		}
		if (ucCurrentBand != ucDesiredBand) {
			ucCurrentBand = ucDesiredBand;
			Msg.SendCmd(ICOMMSG_VFOSELECT, ucDesiredBand);
			return BOOL_FALSE;
		}
		return BOOL_TRUE;	// already desired band
	}

	// No copy constructor or assignment allowed
	// Force compile error if attempted
	ICOM(const ICOM&);
	ICOM& operator=(const ICOM&);

public:
	// General Functions
	ICOM(const RADIOPARAMS& Params,
	     const USHORT unRadioType = ICOM_DEFAULT);
	BOOL	IsReady() { return eReady; }
	VOID	Update();
	~ICOM();

	// Tx Functions
	VOID	SetTxFreq(DOUBLE dFrequency);
	VOID	SetTxMode(MODULATION eMode);

	// Rx Functions
	BOOL	HasUserTunedRx() { return eUserHasTunedRx; }
	VOID	RefreshRxParams();
	DOUBLE	GetRxFreq();
	MODULATION GetRxMode() { return eRxMode; }
	VOID	SetRxFreq(DOUBLE dFrequency);
	VOID	SetRxMode(MODULATION eMode);

};
#endif // end icom.hpp
