/****************************************************************************
Date: 3/8/2002
File: icommsg.cpp	
Description:
	ICOM message handler class 

	This class handles low-level serial io and
	ICOM message encoding and decoding

	public:
	//***** General Functions *****
	ICOMMSG();
	BOOL	Open(USHORT unICOMAddress,
		     UINT	unComPort,
		     UINT	unBitRate,
		     UINT	unIRQ,
		     IOADDRESS	Address);
	VOID	Close();
	~ICOMMSG();

	//***** Send messages to radio *****
	VOID	SendCmd(UCHAR ucCmd);
	VOID	SendCmd(UCHAR ucCmd, UCHAR ucSubCmd);
	VOID	SendWriteFrequency(LONG lFrequency);
	VOID	SendWriteMode(MODULATION eMode);

	//***** Get messages from radio *****
	BOOL	IsReady();
	UCHAR	GetCmd();
	LONG	GetFrequency();	// if GetCmd => ICOMMSG_READFREQ
	MODULATION GetMode();	// if GetCmd => ICOMMSG_READMODE

	***** This is the ICOM Message Structure *****
	UCHAR	ucPreamble1;
	UCHAR	ucPreamble2;
	UCHAR	ucToAddress;
	UCHAR	ucFromAddress;
	UCHAR	ucCommand;
	UCHAR	aVariable[ICOM_MAXVARDATA]; + EOM
History:
	12/6/2001 Initial Version
	3/3/2002 IC-910 enhancements
	3/8/2002 take out IC-910 stuff
*****************************************************************************/
#include "icommsg.hpp"

//***** Message Constants ********************************
static const UCHAR PREAMBLE 	= 0xfe;	// preamble code
static const UCHAR CTRLADDRESS	= 0xe0;	// controller address
static const UCHAR ENDOFMESSAGE	= 0xfd; // end of message code
static const UCHAR JAMMERCODE	= 0xfc; // message collision detected

//***** Message Byte Counts *******************************
static const UINT BASEMSGSIZE	= 6; 	// basic message 
static const UINT SUBCMDMSGSIZE	= 7; 	// message with Sub command
static const UINT WRITEMODESIZE = 7; 	// write mode message
static const UINT WRITEFREQSIZE = 11;	// write freq message size

//***** Array Indices for fields in i/o buffers ************
static const UINT PREAMBLE1	= 0;	// 1st preamble
static const UINT PREAMBLE2	= 1;	// 2nd Preamble
static const UINT RXADDRESS 	= 2;	// receive (destination) address
static const UINT TXADDRESS 	= 3;	// transmit (source) address
static const UINT CMD		= 4;	// cmd opcode
static const UINT DATA		= 5;	// data field start

//***** COM PORT PARAMETERS ******************************************
static const TIMEOUT		= 1;	// 1 second response timer
static const UINT BITRATE	= 1200; // default bit rate
static const UINT DATABITS	= 8;
static const UINT STOPBITS	= 2;
static const UARTPARITY PARITY	= UARTPARITY_NONE;

//***** Shared COM port driver ****************************************
COMDRV ICOMMSG::Comport;

//***** Conversion tables between ICOM mode and MODULATION
const MODULATION ICOMMSG::aGetModulation[] = {
	MODULATION_LSB,	// 0
	MODULATION_USB,	// 1
	MODULATION_AM,	// 2
	MODULATION_CW,	// 3
	MODULATION_FSK,	// 4
	MODULATION_FM,	// 5
	MODULATION_FM	// 6
};

const UCHAR ICOMMSG::aGetICOMMode[] = {
	1,	// MODULATION_NA (default USB)
	0,	// MODULATION_LSB
	1,	// MODULATION_USB
	3,	// MODULATION_CW
	5,	// MODULATION_FM
	2,	// MODULATION_AM
	4	// MODULATION_FSK
};

/****************************************************************************
Function: SendMessage()					PRIVATE
Description:
	Add the command byte and the EOM to a message in the
	Output Buffer and Send it to the radio.
	Then start message timout timer.
Parameters:
	UCHAR 	ucCmd		// command byte
	USHORT 	unByteCount	// total bytes in message
Returns:
****************************************************************************/
VOID	ICOMMSG::SendMessage(UCHAR ucCmd, USHORT unByteCount) {

	aOutputBuffer[CMD] = ucCmd;
	aOutputBuffer[unByteCount - 1] = ENDOFMESSAGE;

	unOutputBytes = unByteCount;
	Comport.Write(aOutputBuffer, unOutputBytes);

	Timer.Start(TIMEOUT);
	return;
}


/****************************************************************************
Function: Resend();				PRIVATE
Description:
	Resend last message and restart timeout timer
Parameters:
Returns:
****************************************************************************/
VOID	ICOMMSG::Resend() {
	Comport.Write(aOutputBuffer, unOutputBytes);
	Timer.Start(TIMEOUT);
	return;
}


/****************************************************************************
Function: ICOMMSG()
Description: constructor
Parameters:
Returns:
*****************************************************************************/
ICOMMSG::ICOMMSG() : unInputBytes(0), unOutputBytes(0), Timer() { }



/****************************************************************************
Function: Open()
Description: Open ICOMMSG for i/o
Parameters:
Returns:	BOOL_TRUE	if operational
		BOOL_FALSE	if couldn't open COM port
*****************************************************************************/
BOOL	ICOMMSG::Open(	USHORT	unICOMAddress,
		 	UINT	unComPort,
		 	UINT	unBitRate,
		 	UINT	unIRQ,
		 	IOADDRESS Address) {
	
	// Preset fixed fields in output buffer
	aOutputBuffer[PREAMBLE1] = PREAMBLE;
	aOutputBuffer[PREAMBLE2] = PREAMBLE;
	aOutputBuffer[RXADDRESS] = (UCHAR) unICOMAddress; // radio address
	aOutputBuffer[TXADDRESS] = CTRLADDRESS; // controller address
	
	// Open com port
	return	Comport.Open(	unComPort, 
		     		(unBitRate ? unBitRate : BITRATE),
				DATABITS,
				PARITY,
				STOPBITS,
				unIRQ,
				Address);
}


/****************************************************************************
Function: Close()
Description: Close ICOMMSG for serial i/io
	     Closes COM port
Parameters:
Returns:
*****************************************************************************/
VOID	ICOMMSG::Close() {
	Comport.Close();
}


/****************************************************************************
Function: ~ICOMMSG()
Description: destructor
Parameters:
Returns:
*****************************************************************************/
ICOMMSG::~ICOMMSG() {
	Close();
}


/****************************************************************************
Function: SendCmd()
Description:
	Send a basic command to the radio (no DATA field.)
	Add the command bytes and the EOM to
	the Output Buffer and send it to the radio.
	Start message timout timer.
Parameters:
	UCHAR ucCmd	// command byte
Returns:
****************************************************************************/
VOID	ICOMMSG::SendCmd(UCHAR ucCmd) {
	SendMessage(ucCmd,BASEMSGSIZE);
	return;
}


/****************************************************************************
Function: SendCmd()
Description:
	Send a basic command and sub command to the radio.
	Add the command and sub command bytes and the EOM to
	the Output Buffer and send it to the radio.
	Start message timout timer.
Parameters:
	UCHAR ucCmd	// command byte
	UCHAR ucSubCmd	// sub command byte
Returns:
****************************************************************************/
VOID	ICOMMSG::SendCmd(UCHAR ucCmd, UCHAR ucSubCmd) {
	aOutputBuffer[DATA] = ucSubCmd;
	SendMessage(ucCmd,SUBCMDMSGSIZE);
	return;
}


/****************************************************************************
Function: SendWriteFrequency()
Description:
	Create a "Write Frequency" message and send to radio
Parameters:
	LONG	lFrequency
Returns:
****************************************************************************/
VOID	ICOMMSG::SendWriteFrequency(LONG lFrequency) {

	// First Encode frequency in BCD into output buffer
	// Start with last freq byte in buffer (MSB) and work backwards
	// DATA field is 5 bytes of BCD.
	LONG	lMultiplier = 1000000000L; // (1GHz)
	LONG	lTmp;	// temp accumulator
	for (INT i = 4; i >= 0; --i) {

		// Do upper nibble
		lTmp = lFrequency / lMultiplier; // integer part only
		aOutputBuffer[DATA+i] = (UCHAR) (lTmp << 4);
		lFrequency -= lTmp * lMultiplier;
		lMultiplier /= 10;

		// Do least sig nibble
		lTmp = lFrequency / lMultiplier; // integer part only
		aOutputBuffer[DATA+i] |= (UCHAR) lTmp;
		lFrequency -= lTmp * lMultiplier;
		lMultiplier /= 10;
	}
	
	// Now Send write Freq message 
	SendMessage(ICOMMSG_WRITEFREQ, WRITEFREQSIZE); 
	return;
}


/****************************************************************************
Function: SendWriteMode();			PRIVATE
Description:
	Create a "Write Mode" message and send to radio
Parameters:
	MODULATION	eMode
Returns:
****************************************************************************/
VOID	ICOMMSG::SendWriteMode(MODULATION eMode) {

	// Get Mode code and put in output buffer
	aOutputBuffer[DATA] = aGetICOMMode[eMode];
	SendMessage(ICOMMSG_WRITEMODE, WRITEMODESIZE); 
	return;
}


/****************************************************************************
Function: IsReady()
Description:
	Process received bytes from radio. When complete
	message is received, returns TRUE.
	Resends last command if no response after TIMEOUT (1 second)
Parameters:
Returns:
	BOOL_TRUE when complete message received
	BOOL_FALSE otherwise
*****************************************************************************/
BOOL	ICOMMSG::IsReady() {

	// If timer expires, resend last message
	if (Timer.IsExpired()) {
		unInputBytes = 0;	// clear any received bytes
		Resend();
		return BOOL_FALSE;
	}

	// Get next message byte
	INT nByte = Comport.Getc();
	if (nByte == COMDRV_EOF) {
		return BOOL_FALSE;		// nothing received
	}

	// Check for Jammer code
	if (nByte == JAMMERCODE) {
		unInputBytes = 0;	// clear any received bytes
		return BOOL_FALSE;
	}

	// Got a new byte, copy it to input buffer
	// But make sure it will fit
	if (unInputBytes <= MAXMSGSIZE) {
		aInputBuffer[unInputBytes++] = (UCHAR) nByte;
	}
	else {	// error - message is too big
		unInputBytes = 0;	// clear any received bytes
		return BOOL_FALSE;
	}

	// Check to see if End Of Message
	if ( nByte != ENDOFMESSAGE) {
		return BOOL_FALSE;		// not done yet
	}

	// END OF MESSAGE - Check params and
	// make sure message is for controller
	if (	(aInputBuffer[PREAMBLE1] != PREAMBLE ) ||
		(aInputBuffer[PREAMBLE2] != PREAMBLE ) ||
		(aInputBuffer[RXADDRESS] != CTRLADDRESS) ) {

		unInputBytes = 0;	// clear any received bytes
		return BOOL_FALSE;	// message not for us (or bad)
	}

	// Message received and ready to process
	Timer.Stop();				// stop message timeout timer
	unInputBytes = 0;			// reset counter
	return BOOL_TRUE;			// Message Ready
}


/****************************************************************************
Function: GetCmd()
Description:
	Gets message command code from received message
	Valid when IsReady() returns TRUE
Parameters:
Returns:
	UCHAR ICOMMSG
*****************************************************************************/
UCHAR ICOMMSG::GetCmd() {
	return aInputBuffer[CMD];
}


/****************************************************************************
Function: GetFrequency()
Description:
	Decodes a BCD frequency in the InputBuffer
	Only valid when the recived message is ICOMMSG_READFREQ
Parameters:
Returns:
	LONG lFrequency
*****************************************************************************/
LONG ICOMMSG::GetFrequency() {

	//***** Convert ICOM BCD to LONG *****************************
	LONG lFrequency = 0;		// accumulator
	LONG lMultiplier = 1;		// decade multiplier

	// Do for all 5 DATA field bytes
	for (INT i = DATA; i < (DATA+5); i++) {
		//***** Get lower nibble *****
		lFrequency += lMultiplier * (aInputBuffer[i] & 0x0f);
		lMultiplier *= 10;

		//***** Get Upper nibble *****
		lFrequency += lMultiplier * ((aInputBuffer[i] & 0xf0) >> 4);
		lMultiplier *= 10;
	}
	return lFrequency;
}


/****************************************************************************
Function: GetMode()
Description:
	Decodes the ICOM mode code and returns MODULATION
	Only valid when the recived message is ICOMMSG_READMODE
Parameters:
Returns:
	MODULATION
*****************************************************************************/
MODULATION ICOMMSG::GetMode() {
	return aGetModulation[aInputBuffer[DATA]];
}
// end icommsg.cpp
