/****************************************************************************
Copyright 1997 Anthony Monteiro

Date: 11/19/97
File: sat.hpp
Description:	SATELLITE CLASS
	
	The SATELLITE class implements satellites with 1 main transponder 
	having both up and down links and a second auxillary transponder
	with just a downlink.

	The Main transponder can be an analog or digital type. The Auxillary 
	transponder can be either a digital type or a second analog 
	type that shares the uplink with the main transponder 
	(like RS-12 Mode K/T with 2 downlinks.) Only the main transponder
	downlink frequency can be set. In 2 analog transponder operation,
	the auxillary tranponder is slaved to the main transponder input
	frequency.

	The Uplink and Downlink are from the standpoint of
	the earth station. For example, GetDownLink() returns the frequency
	that the earth station receiver should use. 

	The velocity of the satellite is from the standpoint of the
	earth station. When the satellite is approaching the earth station,
	its velocity is positive. The SetVelocity() function takes the
	actual satellite velocity normalized to the speed of light (i.e.
	the velocity is unitless.)

	To create SATELLITE objects, call the NewInstance() function with
	the satellite parameters. This creates a satellite object with
	the specified transponder types. To destroy satellite objects,
	use the usual delete() function.

	public:
		static SATELLITE* NewInstance(const SATELLITEPARAMS& Params);

		VOID	SetVelocity(DOUBLE dVelocity)

		DOUBLE	GetDownlink() const
		DOUBLE	GetDownlink2() const
		DOUBLE	GetUplink() const

		MODULATION GetDownlinkMode() const
		MODULATION GetDownlink2Mode() const
		MODULATION GetUplinkMode() const

		VOID	SetDownlink(DOUBLE dFreq)
		VOID	SetDownlinkMode(MODULATION eMode)

		~SATELLITE()

History
	11/19/97 Release 1.0
*****************************************************************************/
#ifndef SAT_HPP
#define SAT_HPP

#include <stddef.h>

#include "loctypes.h"
#include "modulate.hpp"

#include "xpdrbase.hpp"		// Transponder base class

struct SATELLITEPARAMS {
	TRANSPONDERPARAMS	Main;	// first transponder
	TRANSPONDERPARAMS	Aux;	// second transponder
};

class SATELLITE {
private:
	DOUBLE	dDopplerAdjust; // 1 - normalized satellite velocity

	TRANSPONDER* const pTransponder1;
	TRANSPONDER* const pTransponder2;

	// Adjust Frequency to correct for shift FROM satellite
	DOUBLE DopplerAdjustDown(DOUBLE dFreq) const {
		return dFreq * dDopplerAdjust;
	}

	// Adjust Frequency to correct for shift TO satellite
	DOUBLE DopplerAdjustUp(DOUBLE dFreq) const {
		return dFreq / dDopplerAdjust;
	}

	// no copy or assignment allowed
	SATELLITE(const SATELLITE&);
	SATELLITE& operator=(const SATELLITE&);

protected:
	SATELLITE(TRANSPONDER* pTransponder1, TRANSPONDER* pTransponder2) :
		dDopplerAdjust(1.0),
		pTransponder1(pTransponder1),
		pTransponder2(pTransponder2) {

		pTransponder2->SetInputFreq( pTransponder1->GetInputFreq() );
		pTransponder2->SetInputMode( pTransponder1->GetInputMode() );
	}

public:
	static SATELLITE* NewInstance(const SATELLITEPARAMS& Params) {
		TRANSPONDER* pTransponder1 =
			TRANSPONDER::NewInstance(Params.Main);
		TRANSPONDER* pTransponder2 =
			TRANSPONDER::NewInstance(Params.Aux);

		// Make sure transponders were created
		// If either failed, clean-up and return NULL pointer
		if ( (pTransponder1 == NULL) || (pTransponder2 == NULL) ) {
			delete pTransponder1;
			delete pTransponder2;
			return NULL;
		}
		
		return new SATELLITE(pTransponder1, pTransponder2);
	}

	VOID	SetVelocity(DOUBLE dVelocity) {
		dDopplerAdjust = 1.0 - dVelocity;
	}

	DOUBLE	GetDownlink() const {
		return DopplerAdjustDown( pTransponder1->GetOutputFreq() );
	}

	DOUBLE	GetDownlink2() const {
		return DopplerAdjustDown( pTransponder2->GetOutputFreq() );
	}

	DOUBLE	GetUplink() const {
		return DopplerAdjustUp( pTransponder1->GetInputFreq() );
	}

	MODULATION GetDownlinkMode() const {
		return pTransponder1->GetOutputMode();
	}

	MODULATION GetDownlink2Mode() const {
		return pTransponder2->GetOutputMode();
	}

	MODULATION GetUplinkMode() const { 
		return pTransponder1->GetInputMode();
	}

	VOID	SetDownlink(DOUBLE dFreq) {
		// Setting downlink freq on main transponder also sets
		// uplink freq on second transponder to track if it is analog.
		// Digital transponder will just ignore this
		pTransponder1->SetOutputFreq( DopplerAdjustUp(dFreq) );
		pTransponder2->SetInputFreq( pTransponder1->GetInputFreq() );
	}

	VOID	SetDownlinkMode(MODULATION eMode) {
		// Setting downlink mode on main transponder also sets
		// uplink mode on second transponder to track if it is analog.
		// Digital transponder will just ignore this.
		pTransponder1->SetOutputMode(eMode);
		pTransponder2->SetInputMode( pTransponder1->GetInputMode() );
	}

	~SATELLITE() {
		delete pTransponder1;
		delete pTransponder2;
	}

};
#endif // sat.hpp
